﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class PlayerController : TiledObjectBehaviour {
	[System.Serializable]
	public struct WeaponDefinition {
		public GameObject bulletPrefab;
		public bool continousFire;
		public float coolDownTime;
		public AudioSource fireSnd;
		public float initDistance;
		public bool infiniteAmmo;
	};

	const float WALK_FORCE = 15000.0f;
	const float MAX_X_FORCE = 100.0f;
	const float JUMP_FORCE = -4000.0f;
	const float JETPACK_FORCE = -10000.0f;
	const float MIN_ANGLE = -80.0f;
	const float MAX_ANGLE = 80.0f;
	const float ANGLE_CHANGE_SPEED = 200.0f;
	const float FUEL_MAX = 2.0f;

	public string postfix = "";
	public Transform target;
	public int weaponIndex = 0;
	int _ammoCount;
	public int ammoCount {
		get {
			return _ammoCount;
		}
		set {
			_ammoCount = value;
			UpdateText();
		}
	}

	int _playerEnergy;
	public int playerEnergy {
		get {
			return _playerEnergy;
		}
		set {
			_playerEnergy = value;
			if (_playerEnergy <= 0) {
				deathSnd.Play();
				DieAndRespawn();
			} else {
				if (!hitSnd.isPlaying) hitSnd.Play ();
			}
			UpdateText();
		}
	}
	int _killsCount;
	public int killsCount {
		get {
			return _killsCount;
		}
		set {
			_killsCount = value;
			UpdateText();
		}
	}

	public Text txtPlayer;
	public int playerIndex;

	Light light;
	ParticleSystem psJetpack;
	public AudioSource hitSnd, deathSnd, reloadSnd;

	public WeaponDefinition[] weapons;
	float lastTimeWeaponFired = 0;
	float fuelLevel = FUEL_MAX;

	string hAxis,vAxis,btn1,btn2,btn3;
	bool onPlatform = false;
	float angle = 0;
	bool facingLeft = true;
	bool initialized = false;
	// Use this for initialization
	void Start () {
		psJetpack = GetComponent<ParticleSystem>();
		psJetpack.enableEmission = false;
		light = GetComponent<Light>();
		hAxis = "Horizontal" + postfix;
		vAxis = "Vertical" + postfix;
		btn1 = "Fire1" + postfix;
		btn2 = "Fire2" + postfix;
		btn3 = "Fire3" + postfix;


	}

	void Awake() {

	}

	void DieAndRespawn() {
		_playerEnergy = 100;
		weaponIndex = 0;
		ammoCount = 0;
		transform.position = TerrainBehaviour.instance.FindPlayerPlacement();
	}

	void UpdateText() {
		txtPlayer.text = "Player " + playerIndex.ToString() + "\n" +
			"Energy: " + playerEnergy.ToString() + "\n" +
			"Kills: " + killsCount.ToString () + "\n" +
			"Ammo: " + ammoCount.ToString ();

	}

	/*void OnCollisionEnter(Collision other) {
		if (other.gameObject.tag == "Terrain" &&
			other.contacts[0].normal.z < 0.4f) {
			onPlatform = true;
		}
	}

	void OnCollisionExit(Collision other) {
		if (other.gameObject.tag == "Terrain")
		{
			onPlatform = false;
		}
	}*/

	void OnTriggerEnter(Collider other) {
		print ("dupa");
		if (other.tag == "Crate") {
			CrateBehaviour crate = other.GetComponent<CrateBehaviour>();
			if (weaponIndex == crate.weaponIndex) ammoCount += crate.ammoCount; else ammoCount = crate.ammoCount;
			weaponIndex = crate.weaponIndex;
			reloadSnd.Play();
			UpdateText();
			DestroyObject(other.gameObject);
		}
	}

	// Update is called once per frame
	public override void Update () {
		if (!initialized) {
			initialized = true;
			DieAndRespawn();
			
			UpdateText();
		}


		base.Update();
		rigidbody.AddForce(Vector3.left * Input.GetAxis (hAxis) * Time.deltaTime * WALK_FORCE) ;//* (onPlatform ? 1.0f : 0.2f));

		rigidbody.velocity = new Vector3(Mathf.Clamp (rigidbody.velocity.x,-MAX_X_FORCE,MAX_X_FORCE),rigidbody.velocity.y,rigidbody.velocity.z);

		if (Input.GetAxis (hAxis) < -0.1f) {
			facingLeft = true;
		}
		angle = Mathf.Clamp(angle - Input.GetAxis(vAxis) * Time.deltaTime * ANGLE_CHANGE_SPEED,MIN_ANGLE,MAX_ANGLE);
		if (Input.GetAxis (hAxis) > 0.1f) {
			facingLeft = false;
		}


		/*RaycastHit hitInfo;
		onPlatform = rigidbody.SweepTest(Vector3.forward,out hitInfo);

		if (onPlatform && Input.GetButtonDown(btn1)) {
			onPlatform = false;
			rigidbody.AddForce (Vector3.forward * JUMP_FORCE);
		}*/

		if (Input.GetButton(btn1) && fuelLevel > 0) {
			rigidbody.AddForce (Vector3.forward * JETPACK_FORCE * Time.deltaTime);
			fuelLevel -= Time.deltaTime;
			psJetpack.enableEmission = true;
		} else {
			fuelLevel += Time.deltaTime / 2.0f;
			if (fuelLevel > FUEL_MAX) fuelLevel = FUEL_MAX;
			psJetpack.enableEmission = false;
		}


		Vector3 targetOffset = (Quaternion.Euler(0,angle,0)
		                        * Vector3.left);
		if (facingLeft) targetOffset = new Vector3(-targetOffset.x,targetOffset.y,targetOffset.z);
		target.position = transform.position + 
			targetOffset *25.0f + Vector3.up * 100.0f;

		// firing
		WeaponDefinition currentWeapon = weapons[weaponIndex];
		if ( Time.time > lastTimeWeaponFired + currentWeapon.coolDownTime &&
			(
				Input.GetButtonDown(btn2) ||
		    	(currentWeapon.continousFire && Input.GetButton (btn2))
		    )
		    ) {
			GameObject obj = (GameObject)GameObject.Instantiate(currentWeapon.bulletPrefab);
			obj.transform.position = transform.position + targetOffset * currentWeapon.initDistance;
			obj.transform.rotation = Quaternion.Euler(0,facingLeft ? -angle : 180+angle,0);
			obj.GetComponent<BaseBulletBehaviour>().bulletOrigin = this;
			lastTimeWeaponFired = Time.time;
			currentWeapon.fireSnd.Play();

			light.range = 20.0f;

			if (!currentWeapon.infiniteAmmo) {
				ammoCount--;
				if (ammoCount<=0) {
					ammoCount = 0;
					weaponIndex = 0;
				}
				UpdateText();
			}
		}

		light.range = Mathf.Lerp(light.range,0,Time.deltaTime * 2.0f);
	}
}
