﻿using UnityEngine;
using System.Collections;

public class TerrainBehaviour : MonoBehaviour {
	const float CRATE_TIME_INTERVAL = 15.0f;
	public static TerrainBehaviour instance = null;
	public GameObject cratePrefab;
	Terrain terrain;
	int width,height;
	float crateLastTimeInstantiated = 0.0f;
	public float[,] heights;

	public float[,] bigExplosion = CreateTerrainModifier(35);
	public float[,] smallExplosion = CreateTerrainModifier(10);
	public float[,] smallExpansion = CreateTerrainModifier(10);
	// Use this for initialization

	public void Start () {
		instance = this;
		terrain = GetComponent<Terrain>();
		width = terrain.terrainData.heightmapWidth;
		height = terrain.terrainData.heightmapHeight;
		heights = new float[width, height];
		Generate();
		InstantiateCrate();
	}

	void InstantiateCrate() {
		GameObject obj = (GameObject)GameObject.Instantiate(cratePrefab);
		obj.transform.position = FindPlayerPlacement();
		crateLastTimeInstantiated = Time.time;
	}

	static public int TranslateX(float x) { // o_O
		return (int)(x*1.6f + 260);
	}

	static public int TranslateY(float y) { // o_O
		return (int)(y*2.5f + 260);
	}

	public Vector3 FindPlayerPlacement() {
		float x,y;
		do {
			x = Random.Range(-160.0f,+160.0f);
			y = Random.Range(-100.0f,+100.0f);

		} while (heights[TranslateY(y) % width,TranslateX (x) % height] > 0.2f);

		return new Vector3(x,2.5f,y);
	}

	void Generate() {
		float tileSize = 6;
		int shiftX = Random.Range(-1000,1000);
		int shiftY = Random.Range(-1000,1000);



		
		for (int i = 0; i < width; i++)
		{
			for (int k = 0; k < height; k++)
			{
				heights[k, i] = Mathf.PerlinNoise(((float)i / (float)width) * tileSize * 1.6f + shiftX, ((float)k / (float)height) * tileSize+shiftY);///10.0f;
			}
		}

		// sklejanie

		float height1,height2,lerp;
		float maxRange;

		//gora/doul
		maxRange = height * 0.25f;
		for (int i=0;i<width;++i) {
			for(int k=0;k<maxRange;++k) {
				height1 = heights[i,k];
				height2 = heights[i,height-1-k];
				lerp = (1.0f - k/maxRange) / 2.0f;
				heights[i,k] = Mathf.Lerp (height1,height2, lerp);
				heights[i,height-1-k] = Mathf.Lerp (height2,height1, lerp);
			}
		}
		// lewo/prawo

		maxRange = width * 0.25f;
		// lewo prawo
		for (int i=0;i<maxRange;++i) {
			for(int k=0;k<height;++k) {
				height1 = heights[i,k];
				height2 = heights[width-1-i,k];
				lerp = (1.0f - i/maxRange) / 2.0f;
				heights[i,k] = Mathf.Lerp (height1,height2, lerp);
				heights[width-1-i,k] = Mathf.Lerp (height2,height1, lerp);
			}
		}

		terrain.terrainData.SetHeights(0, 0, heights);
	}

	public float ModifyTerrain(int x, int y, float[,] alpha, float mul) {
		int width2 = alpha.GetLength (0);
		int height2 = alpha.GetLength (1);
		x-=width2/2;
		y-=height2/2;
		float totalAmount = 0;
		float alphaTmp;
		float[,] heights3 = new float[width2,height2];
		int width_ = 200;
		for(int i=0;i<width2;++i)
			for(int j=0;j<height2;++j)
		{
			int x2 = ((x+i-60) % width+60); while (x2<0) x2+=width;
			int y2 = (y+j) % height; while (y2<0) y2+=height;

			alphaTmp = alpha[i,j] * mul;
			float tmp = heights[x2,y2] - 0.5f;
			heights3[i,j] = (heights[x2,y2] += alphaTmp);

			if (tmp > 0)
				totalAmount += Mathf.Min(-alphaTmp,tmp);
		}

		terrain.terrainData.SetHeights(0,0, heights); // TODO: optimize?

		return totalAmount;
	}

	static public float[,] CreateTerrainModifier(int radius) {
		int dim = radius*2;
		int dim2 = Mathf.FloorToInt(dim*1.6f);
		float[,] alpha = new float[dim2,dim];
		float sqrMargin = radius*radius;
		float x,y,sqr;
		for(int i=0;i<dim2;++i)
			for(int j=0;j<dim;++j) {
			x = (i/1.6f-radius);
			y = (j-radius);
			sqr = x*x + y*y;
			alpha[i,j] = sqr > sqrMargin ? 0 : (1.0f - sqr/sqrMargin);
		}

		return alpha;
	}

	// Update is called once per frame
	void Update () {
		if (Input.GetKey(KeyCode.Escape)) {
			Application.LoadLevel(Application.loadedLevel);
		}

		if (Input.GetMouseButtonDown(0)) {
			RaycastHit hitInfo;
			if (Physics.Raycast(Camera.main.ScreenPointToRay(Input.mousePosition),out hitInfo)) {
				int x = TranslateX(hitInfo.point.x);
				int y = TranslateY(hitInfo.point.z);
				ModifyTerrain(y,x,bigExplosion,-0.5f);
			}
		}

		if (Time.time > crateLastTimeInstantiated + CRATE_TIME_INTERVAL) {
			InstantiateCrate();
		}
	}
}
